@@ -19,9 +19,11 @@ from cryptography.hazmat.backends.interfaces import (
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.utils import (
-    encode_rfc6979_signature
+    encode_dss_signature
 )
 
+from .fixtures_ec import EC_KEY_SECP384R1
+from ...doubles import DummyKeySerializationEncryption
 from ...utils import (
     load_fips_ecdsa_key_pair_vectors, load_fips_ecdsa_signing_vectors,
     load_kasvs_ecdh_vectors, load_vectors_from_file,
@@ -80,11 +82,6 @@ class DummySignatureAlgorithm(object):
     algorithm = None
 
 
-@utils.register_interface(serialization.KeySerializationEncryption)
-class DummyKeyEncryption(object):
-    pass
-
-
 @pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
 def test_skip_curve_unsupported(backend):
     with pytest.raises(pytest.skip.Exception):
@@ -97,6 +94,12 @@ def test_skip_exchange_algorithm_unsupported(backend):
         _skip_exchange_algorithm_unsupported(backend, ec.ECDH(), DummyCurve())
 
 
+@pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
+def test_skip_ecdsa_vector(backend):
+    with pytest.raises(pytest.skip.Exception):
+        _skip_ecdsa_vector(backend, DummyCurve, hashes.SHA256)
+
+
 def test_ec_numbers():
     numbers = ec.EllipticCurvePrivateNumbers(
         1,
@@ -433,7 +436,7 @@ class TestECDSAVectors(object):
             curve_type()
         ).public_key(backend)
 
-        signature = encode_rfc6979_signature(vector['r'], vector['s'])
+        signature = encode_dss_signature(vector['r'], vector['s'])
 
         verifier = key.verifier(
             signature,
@@ -462,7 +465,7 @@ class TestECDSAVectors(object):
             curve_type()
         ).public_key(backend)
 
-        signature = encode_rfc6979_signature(vector['r'], vector['s'])
+        signature = encode_dss_signature(vector['r'], vector['s'])
 
         verifier = key.verifier(
             signature,
@@ -734,7 +737,7 @@ class TestECSerialization(object):
             key.private_bytes(
                 serialization.Encoding.PEM,
                 serialization.PrivateFormat.TraditionalOpenSSL,
-                DummyKeyEncryption()
+                DummyKeySerializationEncryption()
             )
 
     def test_public_bytes_from_derived_public_key(self, backend):
@@ -844,7 +847,7 @@ class TestECDSAVerification(object):
 
 
 @pytest.mark.requires_backend_interface(interface=EllipticCurveBackend)
-class TestECDHVectors(object):
+class TestECDH(object):
     @pytest.mark.parametrize(
         "vector",
         load_vectors_from_file(
@@ -916,3 +919,19 @@ class TestECDHVectors(object):
             exceptions._Reasons.UNSUPPORTED_EXCHANGE_ALGORITHM
         ):
             key.exchange(None, key.public_key())
+
+    def test_exchange_non_matching_curve(self, backend):
+        _skip_curve_unsupported(backend, ec.SECP256R1())
+        _skip_curve_unsupported(backend, ec.SECP384R1())
+
+        key = load_vectors_from_file(
+            os.path.join(
+                "asymmetric", "PKCS8", "ec_private_key.pem"),
+            lambda pemfile: serialization.load_pem_private_key(
+                pemfile.read().encode(), None, backend
+            )
+        )
+        public_key = EC_KEY_SECP384R1.public_numbers.public_key(backend)
+
+        with pytest.raises(ValueError):
+            key.exchange(ec.ECDH(), public_key)
